@@ -1,7 +1,6 @@
 package NetSNMP::OID;
 
 use strict;
-use warnings;
 use Carp;
 
 require Exporter;
@@ -27,10 +26,6 @@ sub length {
     return $_[0]->{'oidptr'}->length();
 }
 
-sub get_indexes {
-    return $_[0]->{'oidptr'}->get_indexes();
-}
-
 sub append {
     my $this = shift;
     my $str = shift;
@@ -92,7 +87,7 @@ use vars qw(@ISA %EXPORT_TAGS @EXPORT_OK @EXPORT $VERSION $AUTOLOAD);
 	snmp_oid_compare
         compare
 );
-$VERSION = '5.0905';
+$VERSION = '5.1';
 
 sub new {
     my $type = shift;
@@ -142,8 +137,6 @@ sub to_array($) {
     return $self->{oidptr}->to_array();
 }
 
-sub DESTROY {}
-
 sub AUTOLOAD {
     # This AUTOLOAD is used to 'autoload' constants from the constant()
     # XS function.  If a constant is not found then control is passed
@@ -152,8 +145,7 @@ sub AUTOLOAD {
     my $constname;
     ($constname = $AUTOLOAD) =~ s/.*:://;
     croak "& not defined" if $constname eq 'constant';
-    my $val;
-    ($!, $val) = constant($constname);
+    my $val = constant($constname, @_ ? $_[0] : 0);
     if ($! != 0) {
 	if ($! =~ /Invalid/ || $!{EINVAL}) {
 	    $AutoLoader::AUTOLOAD = $AUTOLOAD;
@@ -184,6 +176,7 @@ bootstrap NetSNMP::OID $VERSION;
 
 1;
 __END__
+# Below is stub documentation for your module. You better edit it!
 
 =head1 NAME
 
@@ -217,9 +210,6 @@ NetSNMP::OID - Perl extension for manipulating OIDs
   $len = $oid3->length();
   # -> 7
 
-  # retrieving indexes from an oid:
-  $arrayref = $tableoid->get_indexes()
-
 =head1 DESCRIPTION
 
 The NetSNMP::OID module is a simple wrapper around a C-based net-snmp
