@@ -1,16 +1,13 @@
-# SNMP.pm -- Perl 5 interface to the Net-SNMP toolkit
+# SNMP.pm -- Perl 5 interface to the net-snmp toolkit
 #
-# written by G. S. Marzot (marz@users.sourceforge.net)
+# written by G. S. Marzot (gmarzot@nortelnetworks.com)
 #
-#     Copyright (c) 1995-2006 G. S. Marzot. All rights reserved.
+#     Copyright (c) 1995-2000 G. S. Marzot. All rights reserved.
 #     This program is free software; you can redistribute it and/or
 #     modify it under the same terms as Perl itself.
 
 package SNMP;
-$VERSION = '5.0403';   # current release version number
-
-use strict;
-use warnings;
+$VERSION = '5.1';   # current release version number
 
 require Exporter;
 require DynaLoader;
@@ -49,12 +46,11 @@ use NetSNMP::default_store (':all');
 );
 
 sub AUTOLOAD {
-    no strict;
     # This AUTOLOAD is used to 'autoload' constants from the constant()
     # XS function.  If a constant is not found then control is passed
     # to the AUTOLOAD in AutoLoader.
     my($val,$pack,$file,$line);
-    my $constname;
+    local($constname);
     ($constname = $AUTOLOAD) =~ s/.*:://;
     # croak "&$module::constant not defined" if $constname eq 'constant';
     $val = constant($constname, @_ ? $_[0] : 0);
@@ -78,23 +74,15 @@ bootstrap SNMP;
 # Preloaded methods go here.
 
 # Package variables
-tie $SNMP::debugging,         'SNMP::DEBUGGING';
-tie $SNMP::debug_internals,   'SNMP::DEBUG_INTERNALS';
-tie $SNMP::dump_packet,       'SNMP::DUMP_PACKET';
-tie %SNMP::MIB,               'SNMP::MIB';
-tie $SNMP::save_descriptions, 'SNMP::MIB::SAVE_DESCR';
-tie $SNMP::replace_newer,     'SNMP::MIB::REPLACE_NEWER';
-tie $SNMP::mib_options,       'SNMP::MIB::MIB_OPTIONS';
+tie $SNMP::debugging, SNMP::DEBUGGING;
+tie $SNMP::debug_internals, SNMP::DEBUG_INTERNALS;
+tie $SNMP::dump_packet, SNMP::DUMP_PACKET;
+tie %SNMP::MIB, SNMP::MIB;
+tie $SNMP::save_descriptions, SNMP::MIB::SAVE_DESCR;
+tie $SNMP::replace_newer, SNMP::MIB::REPLACE_NEWER;
 
 %SNMP::V3_SEC_LEVEL_MAP = (noAuthNoPriv => 1, authNoPriv => 2, authPriv =>3);
 
-use vars qw(
-  $auto_init_mib $use_long_names $use_sprint_value $use_enums
-  $use_numeric %MIB $verbose $debugging $dump_packet $save_descriptions
-  $best_guess $non_increasing $replace_newer %session_params
-  $debug_internals $mib_options
-);
-
 $auto_init_mib = 1; # enable automatic MIB loading at session creation time
 $use_long_names = 0; # non-zero to prefer longer mib textual identifiers rather
                    # than just leaf indentifiers (see translateObj)
@@ -127,8 +115,6 @@ $save_descriptions = 0; #tied scalar to control saving descriptions during
 $best_guess = 0;  # determine whether or not to enable best-guess regular
                   # expression object name translation.  1 = Regex (-Ib),
 		  # 2 = random (-IR)
-$non_increasing = 0; # stop polling with an "OID not increasing"-error
-                     # when an OID does not increases in bulkwalk.
 $replace_newer = 0; # determine whether or not to tell the parser to replace
                     # older MIB modules with newer ones when loading MIBs.
                     # WARNING: This can cause an incorrect hierarchy.
@@ -209,7 +195,7 @@ sub translateObj {
    SNMP::init_snmp("perl");
    my $obj = shift;
    my $temp = shift;
-   my $include_module_name = shift || "0";
+   my $include_module_name = shift;
    my $long_names = $temp || $SNMP::use_long_names;
 
    return undef if not defined $obj;
@@ -392,37 +378,6 @@ sub _tie {
     tie($_[0],$_[1],$_[2],$_[3]);
 }
 
-sub split_vars {
-    # This sub holds the regex that is used throughout this module  
-    #  to parse the base part of an OID from the IID.
-    #  eg: portName.9.30 -> ['portName','9.30'] 
-    my $vars = shift;
-
-    # The regex was changed to this simple form by patch 722075 for some reason.
-    # Testing shows now (2/05) that it is not needed, and that the long expression 
-    # works fine.  AB
-    # my ($tag, $iid) = ($vars =~ /^(.*?)\.?(\d+)+$/);
-    
-    # These following two are the same.  Broken down for easier maintenance
-    # my ($tag, $iid) = ($vars =~ /^((?:\.\d+)+|(?:\w+(?:\-*\w+)+))\.?(.*)$/);
-    my ($tag, $iid) =
-        ($vars =~ /^(               # Capture $1
-                    # 1. either this 5.5.5.5
-                     (?:\.\d+)+     # for grouping, won't increment $1
-                    |
-                    # 2. or asdf-asdf-asdf-asdf
-                     (?:            # grouping again
-                        \w+         # needs some letters followed by
-                        (?:\-*\w+)+ #  zero or more dashes, one or more letters
-                     )
-                    )
-                    \.?             # optionally match a dot
-                    (.*)            # whatever is left in the string is our iid ($2)
-                   $/x
-    );
-    return [$tag,$iid];
-}
-
 package SNMP::Session;
 
 sub new {
@@ -497,19 +452,13 @@ sub new {
        $this->{Context} ||= 
 	   NetSNMP::default_store::netsnmp_ds_get_string(NetSNMP::default_store::NETSNMP_DS_LIBRARY_ID(), 
 		         NetSNMP::default_store::NETSNMP_DS_LIB_CONTEXT()) || '';
-       $this->{AuthProto} ||= 'DEFAULT'; # defaults to the library's default
+       $this->{AuthProto} ||= 'MD5'; # defaults XXX
        $this->{AuthPass} ||=
        NetSNMP::default_store::netsnmp_ds_get_string(NetSNMP::default_store::NETSNMP_DS_LIBRARY_ID(), 
 		     NetSNMP::default_store::NETSNMP_DS_LIB_AUTHPASSPHRASE()) ||
        NetSNMP::default_store::netsnmp_ds_get_string(NetSNMP::default_store::NETSNMP_DS_LIBRARY_ID(), 
 		     NetSNMP::default_store::NETSNMP_DS_LIB_PASSPHRASE()) || '';
-
-       $this->{AuthMasterKey} ||= '';
-       $this->{PrivMasterKey} ||= '';
-       $this->{AuthLocalizedKey} ||= '';
-       $this->{PrivLocalizedKey} ||= '';
-
-       $this->{PrivProto} ||= 'DEFAULT';  # defaults to hte library's default
+       $this->{PrivProto} ||= 'DES';  # defaults XXX
        $this->{PrivPass} ||=
        NetSNMP::default_store::netsnmp_ds_get_string(NetSNMP::default_store::NETSNMP_DS_LIBRARY_ID(), 
 		     NetSNMP::default_store::NETSNMP_DS_LIB_PRIVPASSPHRASE()) ||
@@ -533,15 +482,7 @@ sub new {
 						$this->{PrivPass},
 						$this->{EngineBoots},
 						$this->{EngineTime},
-						$this->{AuthMasterKey},
-						length($this->{AuthMasterKey}),
-						$this->{PrivMasterKey},
-						length($this->{PrivMasterKey}),
-						$this->{AuthLocalizedKey},
-						length($this->{AuthLocalizedKey}),
-						$this->{PrivLocalizedKey},
-						length($this->{PrivLocalizedKey}),
-					       );
+						);
    }
    unless ($this->{SessPtr}) {
        warn("unable to create session") if $SNMP::verbose;
@@ -550,14 +491,11 @@ sub new {
 
    SNMP::initMib($SNMP::auto_init_mib); # ensures that *some* mib is loaded
 
-   $this->{UseLongNames} = $SNMP::use_long_names 
-       unless exists $this->{UseLongNames};
-   $this->{UseSprintValue} = $SNMP::use_sprint_value 
-       unless exists $this->{UseSprintValue};
-   $this->{BestGuess} = $SNMP::best_guess unless exists $this->{BestGuess};
-   $this->{NonIncreasing} ||= $SNMP::non_increasing;
-   $this->{UseEnums} = $SNMP::use_enums unless exists $this->{UseEnums};
-   $this->{UseNumeric} = $SNMP::use_numeric unless exists $this->{UseNumeric};
+   $this->{UseLongNames} ||= $SNMP::use_long_names;
+   $this->{UseSprintValue} ||= $SNMP::use_sprint_value;
+   $this->{BestGuess} ||= $SNMP::best_guess;
+   $this->{UseEnums} ||= $SNMP::use_enums;
+   $this->{UseNumeric} ||= $SNMP::use_numeric;
 
    # Force UseLongNames if UseNumeric is in use.
    $this->{UseLongNames}++  if $this->{UseNumeric};
@@ -571,7 +509,7 @@ sub update {
 # host, community name change, timeout, retry changes etc. Unfortunately not
 # working yet because some updates (the address in particular) need to be
 # done on the internal session pointer which cannot be fetched w/o touching
-# globals at this point which breaks win32. A patch to the net-snmp toolkit
+# globals at this point which breaks win32. A patch to the ucd-snmp toolkit
 # is needed
    my $this = shift;
    my ($name, $aliases, $host_type, $len, $thisaddr);
@@ -579,14 +517,11 @@ sub update {
 
    @$this{keys %new_fields} = values %new_fields;
 
-   $this->{UseLongNames} = $SNMP::use_long_names 
-       unless exists $this->{UseLongNames};
-   $this->{UseSprintValue} = $SNMP::use_sprint_value 
-       unless exists $this->{UseSprintValue};
-   $this->{BestGuess} = $SNMP::best_guess unless exists $this->{BestGuess};
-   $this->{NonIncreasing} ||= $SNMP::non_increasing;
-   $this->{UseEnums} = $SNMP::use_enums unless exists $this->{UseEnums};
-   $this->{UseNumeric} = $SNMP::use_numeric unless exists $this->{UseNumeric};
+   $this->{UseLongNames} ||= $SNMP::use_long_names;
+   $this->{UseSprintValue} ||= $SNMP::use_sprint_value;
+   $this->{BestGuess} ||= $SNMP::best_guess;
+   $this->{UseEnums} ||= $SNMP::use_enums;
+   $this->{UseNumeric} ||= $SNMP::use_numeric;
 
    # Force UseLongNames if UseNumeric is in use.
    $this->{UseLongNames}++  if $this->{UseNumeric};
@@ -617,11 +552,10 @@ sub set {
      $varbind_list_ref = [$vars];
      $varbind_list_ref = $vars if ref($$vars[0]) =~ /ARRAY/;
    } else {
-     #$varbind_list_ref = [[$tag, $iid, $val]];
-     my $split_vars = SNMP::split_vars($vars);
+     # my ($tag, $iid) = ($vars =~ /^((?:\.\d+)+|(?:\w+(?:\-*\w+)+))\.?(.*)$/);
+     my ($tag, $iid) = ($vars =~ /^(.*?)\.?(\d+)+$/);     
      my $val = shift;
-     push @$split_vars,$val;
-     $varbind_list_ref = [$split_vars];
+     $varbind_list_ref = [[$tag, $iid, $val]];
    }
    my $cb = shift;
 
@@ -641,7 +575,9 @@ sub get {
      $varbind_list_ref = [$vars];
      $varbind_list_ref = $vars if ref($$vars[0]) =~ /ARRAY/;
    } else {
-     $varbind_list_ref = [SNMP::split_vars($vars)];
+     # my ($tag, $iid) = ($vars =~ /^((?:\.\d+)+|(?:\w+(?:\-*\w+)+))\.?(.*)$/);
+     my ($tag, $iid) = ($vars =~ /^(.*?)\.?(\d+)+$/);
+     $varbind_list_ref = [[$tag, $iid]];
    }
 
    my $cb = shift;
@@ -652,277 +588,100 @@ sub get {
 }
 
 
-my $have_netsnmp_oid = eval { require NetSNMP::OID; };
+
 sub gettable {
 
-    #
-    # getTable
-    # --------
-    #
-    # Get OIDs starting at $table_oid, and continue down the tree
-    # until we get to an OID which does not start with $table_oid,
-    # i.e. we have reached the end of this table.
-    #
+	#
+	# getTable
+	# --------
+	#
+	# Get OIDs starting at $table_oid, and continue down the tree
+	# until we get to an OID which does not start with $table_oid,
+	# i.e. we have reached the end of this table.
+	#
 
-    my $state;
+	my $this = shift;
+	my $root_oid = shift;
 
-    my ($this, $root_oid, @options) = @_;
-    $state->{'options'} = {@options};
-    my ($textnode, $varbinds, $vbl, $res, $repeat);
 
-    # translate the OID into numeric form if its not
-    if ($root_oid !~ /^[\.0-9]+$/) {
-	$textnode = $root_oid;
-	$root_oid = SNMP::translateObj($root_oid);
-    } else {
-	$textnode = SNMP::translateObj($root_oid);
-    }
+	# Check if the root OID is a VarList
 
-    # bail if we don't have a valid oid.
-    return if (!$root_oid);
+	if(ref($root_oid) =~ /SNMP::VarList/) {
 
-    # deficed if we're going to parse indexes
-    my $parse_indexes = (defined($state->{'options'}{'noindexes'})) ? 
-      0 : $have_netsnmp_oid;
+		$table_root_oid = $root_oid;
 
-    # get the list of columns we should look at.
-    my @columns;
-    if (!$state->{'options'}{'columns'}) {
-	if ($textnode) {
-	    my %indexes;
+	} elsif(ref($root_oid) =~ /SNMP::Varbind/) {
 
-	    if ($parse_indexes) {
-		# get indexes
-		my @indexes =
-		  @{$SNMP::MIB{$textnode}{'children'}[0]{'indexes'} || [] };
-		# quick translate into a hash
-		map { $indexes{$_} = 1; } @indexes;
-	    }
+		$table_root_oid = [$root_oid];
 
-	    # calculate the list of accessible columns that aren't indexes
-	    my $children = $SNMP::MIB{$textnode}{'children'}[0]{'children'};
-	    foreach my $c (@$children) {
-		push @{$state->{'columns'}},
-		  $root_oid . ".1." . $c->{'subID'}
-		    if (!$indexes{$c->{'label'}});
-	    }
-	    if ($#{$state->{'columns'}} == -1) {
-		# some tables are only indexes, and we need to walk at
-		# least one column.  We pick the last.
-		push @{$state->{'columns'}}, $root_oid . ".1." .
-		  $children->[$#$children]{'subID'}
-		  if ref($state) eq 'HASH' and ref($children) eq 'HASH';
-	    }
-	}
-    } else {
-	# XXX: requires specification in numeric OID...  ack.!
-	@{$state->{'columns'}} = @{$state->{'options'}{'columns'}};
+	} elsif(ref($root_oid) =~ /ARRAY/) {
 
-	# if the columns aren't numeric, we need to turn them into
-	# numeric columns...
-	map {
-	    if ($_ !~ /\.1\.3/) {
-		$_ = $SNMP::MIB{$_}{'objectID'};
-	    }
-	} @{$state->{'columns'}};
-    }
+		$table_root_oid = [$root_oid];
+		$table_root_oid = $root_oid if ref($$root_oid[0]) =~ /ARRAY/;
 
-    # create the initial walking info.
-    foreach my $c (@{$state->{'columns'}}) {
-	push @{$state->{'varbinds'}}, [$c];
-	push @{$state->{'stopconds'}}, $c;
-    }
-
-    if ($#{$state->{'varbinds'}} == -1) {
-	print STDERR "ack: gettable failed to find any columns to look for.\n";
-	return;
-    }
-
-    $vbl = $state->{'varbinds'};
-	
-    my $repeatcount;
-    if ($this->{Version} eq '1' || $state->{'options'}{nogetbulk}) {
-	$state->{'repeatcount'} = 1;
-    } elsif ($state->{'options'}{'repeat'}) {
-	$state->{'repeatcount'} = $state->{'options'}{'repeat'};
-    } elsif ($#{$state->{'varbinds'}} == -1) {
-	$state->{'repeatcount'} = 1;
-    } else {
-	# experimentally determined maybe guess at a best repeat value
-	# 1000 bytes max (safe), 30 bytes average for encoding of the
-	# varbind (experimentally determined to be closer to
-	# 26.  Again, being safe.  Then devide by the number of
-	# varbinds.
-	$state->{'repeatcount'} = int(1000 / 36 / ($#{$state->{'varbinds'}} + 1));
-    }
-
-    #
-    # if we've been configured with a callback, then call the
-    # sub-functions with a callback to our own "next" processing
-    # function (_gettable_do_it).  or else call the blocking method and
-    # call the next processing function ourself.
-    #
-    if ($state->{'options'}{'callback'}) {
-	if ($this->{Version} ne '1' && !$state->{'options'}{'nogetbulk'}) {
-	    $res = $this->getbulk(0, $state->{'repeatcount'}, $vbl,
-				  [\&_gettable_do_it, $this, $vbl,
-				   $parse_indexes, $textnode, $state]);
 	} else {
-	    $res = $this->getnext($vbl,
-				  [\&_gettable_do_it, $this, $vbl,
-				   $parse_indexes, $textnode, $state]);
-	}
-    } else {
-	if ($this->{Version} ne '1' && !$state->{'options'}{'nogetbulk'}) {
-	    $res = $this->getbulk(0, $state->{'repeatcount'}, $vbl);
-	} else {
-	    $res = $this->getnext($vbl);
-	}
-	return $this->_gettable_do_it($vbl, $parse_indexes, $textnode, $state);
-    }
-    return 0;
+
+		my($tag, $iid) = ($root_oid =~ /^((?:\.\d+)+|(?:\w+(?:\-*\w+)+))\.?(.*)$/);
+		$table_root_oid = [[$tag, $iid]];
+
+	};
+
+
+	# Translate the supplied OID in to numeric format for parsing
+
+	$table_root_oid = SNMP::translateObj(@{@{$table_root_oid}[0]}[0]);
+
+
+	my %result_hash;
+	my $end_of_table = 0;
+
+
+	my $table_varbind = new SNMP::Varbind([$table_root_oid]);
+
+	while($this->getnext($table_varbind) && !$this->{ErrorNum}) {
+
+		my $row_oid = SNMP::translateObj(@{$table_varbind}[0]);
+		my $row_text = @{$table_varbind}[0];
+		my $row_index = @{$table_varbind}[1];
+		my $row_value = @{$table_varbind}[2];
+		my $row_type = @{$table_varbind}[3];
+
+		if($row_oid =~ m/($table_root_oid)/) {
+
+			if(($row_type eq "OCTETSTR") && ($row_value =~ m/^\W/)) {
+
+				# If the value returned is an octet string and
+				# includes non-word or non-digit values, unpack
+				# them to produce cleartext
+
+				$row_value = unpack("H*", $row_value);
+
+			} elsif($row_type eq "OBJECTID") {
+
+				# If the value returned is an OID, translate this
+				# back in to a textual OID
+
+				$row_value = SNMP::translateObj($row_value);
+
+			};
+
+
+			# Place the results in a hash
+
+			$result_hash{$row_index}->{$row_text} = $row_value;
+
+		} else {
+
+			last;
+
+		};
+
+	};
+
+	return(\%result_hash);
+
 }
 
-sub _gettable_do_it() {
-    my ($this, $vbl, $parse_indexes, $textnode, $state) = @_;
-
-    my ($res);
-
-    $vbl = $_[$#_] if ($state->{'options'}{'callback'});
-
-    while ($#$vbl > -1 && !$this->{ErrorNum}) {
-	if (($#$vbl + 1) % ($#{$state->{'stopconds'}} + 1) != 0) {
-	    if ($vbl->[$#$vbl][2] ne 'ENDOFMIBVIEW') {
-		# unless it's an end of mib view we didn't get the
-		# proper number of results back.
-		print STDERR "ack: gettable results not appropriate\n";
-	    }
-	    my @k = keys(%{$state->{'result_hash'}});
-	    last if ($#k > -1);  # bail with what we have
-	    return;
-	}
-
-	$state->{'varbinds'} = [];
-	my $newstopconds;
-
-	my $lastsetstart = ($state->{'repeatcount'}-1) * ($#{$state->{'stopconds'}}+1);
-
-	for (my $i = 0; $i <= $#$vbl; $i++) {
-	    my $row_oid = SNMP::translateObj($vbl->[$i][0]);
-	    my $row_text = $vbl->[$i][0];
-	    my $row_index = $vbl->[$i][1];
-	    my $row_value = $vbl->[$i][2];
-	    my $row_type = $vbl->[$i][3];
-
-	    if ($row_oid =~ 
-		/^$state->{'stopconds'}[$i % ($#{$state->{'stopconds'}}+1)]/ &&
-		$row_value ne 'ENDOFMIBVIEW' ){
-
-		if ($row_type eq "OBJECTID") {
-
-		    # If the value returned is an OID, translate this
-		    # back in to a textual OID
-
-		    $row_value = SNMP::translateObj($row_value);
-
-		}
-
-		# Place the results in a hash
-
-		$state->{'result_hash'}{$row_index}{$row_text} = $row_value;
-
-		# continue past this next time
-		if ($i >= $lastsetstart) {
-		    push @$newstopconds,
-		      $state->{'stopconds'}->[$i%($#{$state->{'stopconds'}}+1)];
-		    push @{$state->{'varbinds'}},[$vbl->[$i][0],$vbl->[$i][1]];
-		}
-	    }
-	}
-	if ($#$newstopconds == -1) {
-	    last;
-	}
-	if ($#{$state->{'varbinds'}} == -1) {
-	    print "gettable ack.  shouldn't get here\n";
-	}
-	$vbl = $state->{'varbinds'};
-	$state->{'stopconds'} = $newstopconds;
-
-        #
-        # if we've been configured with a callback, then call the
-        # sub-functions with a callback to our own "next" processing
-        # function (_gettable_do_it).  or else call the blocking method and
-        # call the next processing function ourself.
-        #
-	if ($state->{'options'}{'callback'}) {
-	    if ($this->{Version} ne '1' && !$state->{'options'}{'nogetbulk'}) {
-		$res = $this->getbulk(0, $state->{'repeatcount'}, $vbl,
-				      [\&_gettable_do_it, $this, $vbl,
-				       $parse_indexes, $textnode, $state]);
-	    } else {
-		$res = $this->getnext($vbl,
-				      [\&_gettable_do_it, $this, $vbl,
-				       $parse_indexes, $textnode, $state]);
-	    }
-	    return;
-	} else {
-	    if ($this->{Version} ne '1' && !$state->{'options'}{'nogetbulk'}) {
-		$res = $this->getbulk(0, $state->{'repeatcount'}, $vbl);
-	    } else {
-		$res = $this->getnext($vbl);
-	    }
-	}
-    }
-
-    # finish up
-    _gettable_end_routine($state, $parse_indexes, $textnode);
-
-    # return the hash if no callback was specified
-    if (!$state->{'options'}{'callback'}) {
-	return($state->{'result_hash'});
-    }
-
-    #
-    # if they provided a callback, call it
-    #   (if an array pass the args as well)
-    #
-    if (ref($state->{'options'}{'callback'}) eq 'ARRAY') {
-	my $code = shift @{$state->{'options'}{'callback'}};
-	$code->(@{$state->{'options'}{'callback'}}, $state->{'result_hash'});
-    } else {
-	$state->{'options'}{'callback'}->($state->{'result_hash'});
-    }
-}
-
-sub _gettable_end_routine {
-    my ($state, $parse_indexes, $textnode) = @_;
-    if ($parse_indexes) {
-	my @indexes = @{$SNMP::MIB{$textnode}{'children'}[0]{'indexes'}};
-	my $i;
-	foreach my $trow (keys(%{$state->{'result_hash'}})) {
-	    my $noid = new NetSNMP::OID($state->{'columns'}[0] . "." . $trow);
-	    if (!$noid) {
-		print STDERR "***** ERROR parsing $state->{'columns'}[0].$trow MIB OID\n";
-		next;
-	    }
-	    my $nindexes = $noid->get_indexes();
-	    if (!$nindexes || ref($nindexes) ne 'ARRAY' ||
-		$#indexes != $#$nindexes) {
-		print STDERR "***** ERROR parsing $state->{'columns'}[0].$trow MIB indexes:\n  $noid => " . ref($nindexes) . "\n   [should be an ARRAY]\n  expended # indexes = $#indexes\n";
-		if (ref($nindexes) eq 'ARRAY') {
-		    print STDERR "***** ERROR parsing $state->{'columns'}[0].$trow MIB indexes: " . ref($nindexes) . " $#indexes $#$nindexes\n";
-		}
-		next;
-	    }
-
-	    for ($i = 0; $i <= $#indexes; $i++) {
-		$state->{'result_hash'}{$trow}{$indexes[$i]} = $nindexes->[$i];
-	    }
-	}
-    }
-}
-
-
 sub fget {
    my $this = shift;
    my $vars = shift;
@@ -936,15 +695,17 @@ sub fget {
      $varbind_list_ref = [$vars];
      $varbind_list_ref = $vars if ref($$vars[0]) =~ /ARRAY/;
    } else {
-     $varbind_list_ref = [SNMP::split_vars($vars)];
+     # my ($tag, $iid) = ($vars =~ /^((?:\.\d+)+|(?:\w+(?:\-*\w+)+))\.?(.*)$/);
+     my ($tag, $iid) = ($vars =~ /^(.*?)\.?(\d+)+$/);
+     $varbind_list_ref = [[$tag, $iid]];
    }
 
    my $cb = shift;
 
    SNMP::_get($this, $this->{RetryNoSuch}, $varbind_list_ref, $cb);
 
-   foreach my $varbind (@$varbind_list_ref) {
-     my $sub = $this->{VarFormats}{$varbind->[$SNMP::Varbind::tag_f]} ||
+   foreach $varbind (@$varbind_list_ref) {
+     $sub = $this->{VarFormats}{$varbind->[$SNMP::Varbind::tag_f]} ||
 	 $this->{TypeFormats}{$varbind->[$SNMP::Varbind::type_f]};
      &$sub($varbind) if defined $sub;
      push(@res, $varbind->[$SNMP::Varbind::val_f]);
@@ -966,7 +727,9 @@ sub getnext {
      $varbind_list_ref = [$vars];
      $varbind_list_ref = $vars if ref($$vars[0]) =~ /ARRAY/;
    } else {
-     $varbind_list_ref = [SNMP::split_vars($vars)];
+     # my ($tag, $iid) = ($vars =~ /^((?:\.\d+)+|(?:\w+(?:\-*\w+)+))\.?(.*)$/);
+     my ($tag, $iid) = ($vars =~ /^(.*?)\.?(\d+)+$/);
+     $varbind_list_ref = [[$tag, $iid]];
    }
 
    my $cb = shift;
@@ -989,15 +752,17 @@ sub fgetnext {
      $varbind_list_ref = [$vars];
      $varbind_list_ref = $vars if ref($$vars[0]) =~ /ARRAY/;
    } else {
-     $varbind_list_ref = [SNMP::split_vars($vars)];
+     # my ($tag, $iid) = ($vars =~ /^((?:\.\d+)+|(?:\w+(?:\-*\w+)+))\.?(.*)$/);
+     my ($tag, $iid) = ($vars =~ /^(.*?)\.?(\d+)+$/);
+     $varbind_list_ref = [[$tag, $iid]];
    }
 
    my $cb = shift;
 
    SNMP::_getnext($this, $varbind_list_ref, $cb);
 
-   foreach my $varbind (@$varbind_list_ref) {
-     my $sub = $this->{VarFormats}{$varbind->[$SNMP::Varbind::tag_f]} ||
+   foreach $varbind (@$varbind_list_ref) {
+     $sub = $this->{VarFormats}{$varbind->[$SNMP::Varbind::tag_f]} ||
 	 $this->{TypeFormats}{$varbind->[$SNMP::Varbind::type_f]};
      &$sub($varbind) if defined $sub;
      push(@res, $varbind->[$SNMP::Varbind::val_f]);
@@ -1021,7 +786,9 @@ sub getbulk {
      $varbind_list_ref = [$vars];
      $varbind_list_ref = $vars if ref($$vars[0]) =~ /ARRAY/;
    } else {
-     $varbind_list_ref = [SNMP::split_vars($vars)];
+     # my ($tag, $iid) = ($vars =~ /^((?:\.\d+)+|(?:\w+(?:\-*\w+)+))\.?(.*)$/);
+     my ($tag, $iid) = ($vars =~ /^(.*?)\.?(\d+)+$/);
+     $varbind_list_ref = [[$tag, $iid]];
    }
 
    my $cb = shift;
@@ -1076,7 +843,7 @@ sub bulkwalk {
    return defined($cb) ? $res[0] : \@res;
 }
 
-my %trap_type = (coldStart => 0, warmStart => 1, linkDown => 2, linkUp => 3,
+%trap_type = (coldStart => 0, warmStart => 1, linkDown => 2, linkUp => 3,
 	      authFailure => 4, egpNeighborLoss => 5, specific => 6 );
 sub trap {
 # (v1) enterprise, agent, generic, specific, uptime, <vars>
@@ -1168,7 +935,7 @@ sub inform {
 }
 
 package SNMP::TrapSession;
-@SNMP::TrapSession::ISA = ('SNMP::Session');
+@ISA = ('SNMP::Session');
 
 sub new {
    my $type = shift;
@@ -1183,11 +950,11 @@ sub new {
 
 package SNMP::Varbind;
 
-my $tag_f = 0;
-my $iid_f = 1;
-my $val_f = 2;
-my $type_f = 3;
-my $time_f = 4;
+$tag_f = 0;
+$iid_f = 1;
+$val_f = 2;
+$type_f = 3;
+$time_f = 4;
 
 sub new {
    my $type = shift;
@@ -1309,7 +1076,7 @@ sub FETCH {
     my $key = shift;
 
     if (!defined $this->{$key}) {
-	tie(%{$this->{$key}}, 'SNMP::MIB::NODE', $key) or return undef;
+	tie(%{$this->{$key}}, SNMP::MIB::NODE, $key) or return undef;
     }
     $this->{$key};
 }
@@ -1359,7 +1126,6 @@ my %node_elements =
      defaultValue => 0, # returns default value
      description => 0, # returns DESCRIPTION ($SNMP::save_descriptions must
                     # be set prior to MIB initialization/parsing
-     augments => 0, # textual identifier of augmented object
     );
 
 # sub TIEHASH - implemented in SNMP.xs
@@ -1411,16 +1177,6 @@ sub DELETE {
     ${$_[0]} = 0;
 }
 
-package SNMP::MIB::MIB_OPTIONS;
-
-sub TIESCALAR { my $class = shift; my $val; bless \$val, $class; }
-
-sub FETCH { ${$_[0]}; }
-
-sub STORE { SNMP::_mib_toggle_options($_[1]); ${$_[0]} = $_[1]; }
-
-sub DELETE { SNMP::_mib_toggle_options(0); ${$_[0]} = ''; }
-
 package SNMP;
 END{SNMP::_sock_cleanup() if defined &SNMP::_sock_cleanup;}
 # Autoload methods go after __END__, and are processed by the autosplit prog.
@@ -1430,7 +1186,7 @@ __END__
 
 =head1 NAME
 
-SNMP - The Perl5 'SNMP' Extension Module for the Net-SNMP SNMP package.
+SNMP - The Perl5 'SNMP' Extension Module v3.1.0 for the UCD SNMPv3 Library
 
 =head1 SYNOPSIS
 
@@ -1550,23 +1306,12 @@ default <none>, authentication passphrase
 
 =item PrivProto
 
-default 'DES', privacy protocol [DES, AES] (v3)
+default 'DES', privacy protocol [DES] (v3)
 
 =item PrivPass
 
 default <none>, privacy passphrase (v3)
 
-=item AuthMasterKey
-
-=item PrivMasterKey
-
-=item AuthLocalizedKey
-
-=item PrivLocalizedKey
-
-Directly specified SNMPv3 USM user keys (used if you want to specify
-the keys instead of deriving them from a password as above).
-
 =item VarFormats
 
 default 'undef', used by 'fget[next]', holds an hash
@@ -1626,14 +1371,6 @@ creation. this setting controls how <tags> are parsed.  setting to
 match (defined as -Ib in snmpcmd) and setting to 2 causes a random 
 access lookup (defined as -IR in snmpcmd).
 
-=item NonIncreasing
-
-defaults to the value of SNMP::non_increasing at time of session
-creation. this setting controls if a non-increasing OID during
-bulkwalk will causes an error. setting to 0 causes the default
-behaviour (which may, in very badly performing agents, result in a never-ending loop).
-setting to 1 causes an error (OID not increasing) when this error occur.
-
 =item ErrorStr
 
 read-only, holds the error message assoc. w/ last request
@@ -1768,110 +1505,6 @@ agent to include one additional (unrelated) variable that signals
 the end of the sub-tree, allowing bulkwalk() to determine that
 the request is complete.
 
-=item $results = $sess->gettable(E<lt>TABLE OIDE<gt>, E<lt>OPTIONS<gt>)
-
-This will retrieve an entire table of data and return a hash reference
-to that data.  The returned hash reference will have indexes of the
-OID suffixes for the index data as the key.  The value for each entry
-will be another hash containing the data for a given row.  The keys to
-that hash will be the column names, and the values will be the data.
-
-Example:
-
-  #!/usr/bin/perl
-
-  use SNMP;
-  use Data::Dumper;
-
-  my $s = new SNMP::Session(DestHost => 'localhost');
-
-  print Dumper($s->gettable('ifTable'));
-
-On my machine produces:
-
-  $VAR1 = {
-            '6' => {
-                     'ifMtu' => '1500',
-                     'ifPhysAddress' => 'PV',
-                     # ...
-                     'ifInUnknownProtos' => '0'
-                   },
-            '4' => {
-                     'ifMtu' => '1480',
-                     'ifPhysAddress' => '',
-                     # ...
-                     'ifInUnknownProtos' => '0'
-                   },
-            # ...
-           };
-
-By default, it will try to do as optimized retrieval as possible.
-It'll request multiple columns at once, and use GETBULK if possible.
-A few options may be specified by passing in an I<OPTIONS> hash
-containing various parameters:
-
-=over
-
-=item noindexes => 1
-
-Instructs the code not to parse the indexes and place the results in
-the second hash.  If you don't need the index data, this will be
-faster.
-
-=item columns => [ colname1, ... ]
-
-This specifies which columns to collect.  By default, it will try to
-collect all the columns defined in the MIB table.
-
-=item repeat => I<COUNT>
-
-Specifies a GETBULK repeat I<COUNT>.  IE, it will request this many
-varbinds back per column when using the GETBULK operation.  Shortening
-this will mean smaller packets which may help going through some
-systems.  By default, this value is calculated and attepmts to guess
-at what will fit all the results into 1000 bytes.  This calculation is
-fairly safe, hopefully, but you can either raise or lower the number
-using this option if desired.  In lossy networks, you want to make
-sure that the packets don't get fragmented and lowering this value is
-one way to help that.
-
-=item nogetbulk => 1
-
-Force the use of GETNEXT rather than GETBULK.  (always true for
-SNMPv1, as it doesn't have GETBULK anyway).  Some agents are great
-implementers of GETBULK and this allows you to force the use of
-GETNEXT oprations instead.
-
-=item callback => \&subroutine
-
-=item callback => [\&subroutine, optarg1, optarg2, ...]
-
-If a callback is specified, gettable will return quickly without
-returning results.  When the results are finally retrieved the
-callback subroutine will be called (see the other sections defining
-callback behaviour and how to make use of SNMP::MainLoop which is
-required fro this to work).  An additional argument of the normal hash
-result will be added to the callback subroutine arguments.
-
-Note 1: internally, the gettable function uses it's own callbacks
-which are passed to getnext/getbulk as appropriate.
-
-Note 2: callback support is only available in the SNMP module version
-5.04 and above.  To test for this in code intending to support both
-versions prior to 5.04 and and 5.04 and up, the following should work:
-
-  if ($response = $sess->gettable('ifTable', callback => \&my_sub)) {
-      # got a response, gettable doesn't support callback
-      my_sub($response);
-      $no_mainloop = 1;
-  }
-
-Deciding on whether to use SNMP::MainLoop is left as an excersize to
-the reader since it depends on whether your code uses other callbacks
-as well.
-
-=back
-
 =back
 
 =head1 SNMP::TrapSession
@@ -2108,7 +1741,7 @@ the current version specifier (e.g., 3.1.0)
 default '1', set to 0 to disable automatic reading
 of the MIB upon session creation. set to non-zero
 to call initMib at session creation which will result
-in MIB loading according to Net-SNMP env. variables (see
+in MIB loading according to UCD env. variables (see
 man mib_api)
 
 =item $SNMP::verbose
@@ -2148,15 +1781,6 @@ methods returned as numeric OID's rather than descriptions.
 UseLongNames will be set so that the entire OID will be
 returned.  Set on a per-session basis (see UseNumeric).
 
-=item $SNMP::best_guess
-
-default '0'.  This setting controls how <tags> are 
-parsed.  Setting to 0 causes a regular lookup.  Setting 
-to 1 causes a regular expression match (defined as -Ib 
-in snmpcmd) and setting to 2 causes a random access 
-lookup (defined as -IR in snmpcmd).  Can also be set 
-on a per session basis (see BestGuess)
-
 =item $SNMP::save_descriptions
 
 default '0',set non-zero to have mib parser save
@@ -2256,10 +1880,6 @@ returns 'textualConvention' if defined else 'type'
 
 returns TEXTUAL-CONVENTION
 
-=item TCDescription
-
-returns the TEXTUAL-CONVENTION's DESCRIPTION field.
-
 =item units
 
 returns UNITS
@@ -2281,10 +1901,6 @@ returns array ref of hash ref [{low => num, high => num}, ...]
 returns DESCRIPTION ($SNMP::save_descriptions must
 be set prior to MIB initialization/parsing)
 
-=item reference
-
-returns the REFERENCE clause
-
 =back
 
 =head1 MIB Functions
@@ -2297,7 +1913,7 @@ allows dynamic parsing of the mib and explicit
 specification of mib file independent of enviroment
 variables. called with no args acts like initMib,
 loading MIBs indicated by environment variables (see
-Net-SNMP mib_api docs). passing non-zero second arg
+ucd mib_api docs). passing non-zero second arg
 forces previous mib to be freed and replaced
 B<(Note: second arg not working since freeing previous
 Mib is more involved than before)>.
@@ -2307,7 +1923,7 @@ Mib is more involved than before)>.
 calls library init_mib function if Mib not already
 loaded - does nothing if Mib already loaded. will
 parse directories and load modules according to
-environment variables described in Net-SNMP documentations.
+environment variables described in UCD documentations.
 (see man mib_api, MIBDIRS, MIBS, MIBFILE(S), etc.)
 
 =item &SNMP::addMibDirs(<dir>,...)
@@ -2336,22 +1952,18 @@ all known modules to be loaded.
 
 B<*Not Implemented*>
 
-=item &SNMP::translateObj(<var>[,arg,[arg]])
+=item &SNMP::translateObj(<var>[,arg])
 
-will convert a text obj tag to an OID and vice-versa.
-Any iid suffix is retained numerically.  Default
-behaviour when converting a numeric OID to text
-form is to return leaf identifier only 
-(e.g.,'sysDescr') but when $SNMP::use_long_names 
-is non-zero or a non-zero second arg is supplied it 
-will return a longer textual identifier.  An optional 
-third argument of non-zero will cause the module name 
-to be prepended to the text name (e.g. 
-'SNMPv2-MIB::sysDescr').  When converting a text obj, 
-the $SNMP::best_guess option is used.  If no Mib is 
-loaded when called and $SNMP::auto_init_mib is enabled 
-then the Mib will be loaded. Will return 'undef' upon 
-failure.
+will convert a text obj tag to an OID and
+vice-versa. any iid suffix is retained numerically.
+default behaviour when converting a numeric OID
+to text form is to return leaf indentifier only
+(e.g.,'sysDescr') but when $SNMP::use_long_names
+is non-zero or a non-zero second arg is supplied
+will return longer textual identifier. If no Mib
+is loaded when called and $SNMP::auto_init_mib is
+enabled then the Mib will be loaded. Will return
+'undef' upon failure.
 
 =item &SNMP::getType(<var>)
 
@@ -2417,10 +2029,10 @@ SNMP::TrapSession::trap
 If problems occur there are number areas to look at to narrow down the
 possibilities.
 
-The first step should be to test the Net-SNMP installation
+The first step should be to test the UCD SNMP installation
 independently from the Perl5 SNMP interface.
 
-Try running the apps from the Net-SNMP distribution.
+Try running the apps from the UCD SNMP distribution.
 
 Make sure your agent (snmpd) is running and properly configured with
 read-write access for the community you are using.
@@ -2428,16 +2040,16 @@ read-write access for the community you are using.
 Ensure that your MIBs are installed and enviroment variables are set
 appropriately (see man mib_api)
 
-Be sure to remove old net-snmp installations and ensure headers and
+Be sure to remove old ucd-snmp installations and ensure headers and
 libraries from old CMU installations are not being used by mistake.
 
 If the problem occurs during compilation/linking check that the snmp
-library being linked is actually the Net-SNMP library (there have been
+library being linked is actually the UCD SNMP library (there have been
 name conflicts with existing snmp libs).
 
 Also check that the header files are correct and up to date.
 
-Sometimes compiling the Net-SNMP library with
+Sometimes compiling the UCD SNMP library with
 'position-independent-code' enabled is required (HPUX specifically).
 
 If you cannot resolve the problem you can post to
@@ -2445,9 +2057,9 @@ comp.lang.perl.modules or
 net-snmp-users@net-snmp-users@lists.sourceforge.net
 
 please give sufficient information to analyze the problem (OS type,
-versions for OS/Perl/Net-SNMP/compiler, complete error output, etc.)
+versions for OS/Perl/UCD/compiler, complete error output, etc.)
 
-=head1 Acknowledgements
+=head1 Acknowledments
 
 Many thanks to all those who supplied patches, suggestions and
 feedback.
@@ -2467,13 +2079,7 @@ feedback.
  Wayne Marquette
  Scott Schumate
  Michael Slifcak
- Srivathsan Srinivasagopalan
- Bill Fenner
- Jef Peeraer
- Daniel Hagerty
- Karl "Rat" Schilke and Electric Lightwave, Inc.
  Perl5 Porters
- Alex Burger
 
 Apologies to any/all who's patch/feature/request was not mentioned or
 included - most likely it was lost when paying work intruded on my
@@ -2495,5 +2101,4 @@ bugs, comments, questions to net-snmp-users@lists.sourceforge.net
      Rights Reserved.  This program is free software; you can
      redistribute it and/or modify it under the same terms as Perl
      itself.
-
 =cut
