@@ -1,7 +1,6 @@
 package NetSNMP::default_store;
 
 use strict;
-use warnings;
 use Carp;
 
 require Exporter;
@@ -54,27 +53,11 @@ use vars qw(@ISA %EXPORT_TAGS @EXPORT_OK @EXPORT $VERSION $AUTOLOAD);
 				   NETSNMP_DS_LIB_HAVE_READ_PREMIB_CONFIG
 				   NETSNMP_DS_LIB_HAVE_READ_CONFIG
 				   NETSNMP_DS_LIB_QUICKE_PRINT
-				   NETSNMP_DS_LIB_DONT_PRINT_UNITS
-				   NETSNMP_DS_LIB_NO_DISPLAY_HINT
-				   NETSNMP_DS_LIB_16BIT_IDS
-				   NETSNMP_DS_LIB_DONT_PERSIST_STATE
-				   NETSNMP_DS_LIB_2DIGIT_HEX_OUTPUT
-				   NETSNMP_DS_LIB_IGNORE_NO_COMMUNITY
-				   NETSNMP_DS_LIB_DISABLE_PERSISTENT_LOAD
-				   NETSNMP_DS_LIB_DISABLE_PERSISTENT_SAVE
-				   NETSNMP_DS_LIB_APPEND_LOGFILES
-                                   NETSNMP_DS_LIB_CLIENT_ADDR_USES_PORT
 				   NETSNMP_DS_LIB_MIB_WARNINGS
 				   NETSNMP_DS_LIB_SECLEVEL
 				   NETSNMP_DS_LIB_SNMPVERSION
 				   NETSNMP_DS_LIB_DEFAULT_PORT
 				   NETSNMP_DS_LIB_OID_OUTPUT_FORMAT
-				   NETSNMP_DS_LIB_STRING_OUTPUT_FORMAT
-				   NETSNMP_DS_LIB_HEX_OUTPUT_LENGTH
-				   NETSNMP_DS_LIB_SERVERSENDBUF
-				   NETSNMP_DS_LIB_SERVERRECVBUF
-				   NETSNMP_DS_LIB_CLIENTSENDBUF
-				   NETSNMP_DS_LIB_CLIENTRECVBUF
 				   NETSNMP_DS_SNMP_VERSION_1
 				   NETSNMP_DS_SNMP_VERSION_2c
 				   NETSNMP_DS_SNMP_VERSION_3
@@ -90,18 +73,6 @@ use vars qw(@ISA %EXPORT_TAGS @EXPORT_OK @EXPORT $VERSION $AUTOLOAD);
 				   NETSNMP_DS_LIB_CONFIGURATION_DIR
 				   NETSNMP_DS_LIB_SECMODEL
 				   NETSNMP_DS_LIB_MIBDIRS
-				   NETSNMP_DS_LIB_OIDSUFFIX
-				   NETSNMP_DS_LIB_OIDPREFIX
-				   NETSNMP_DS_LIB_CLIENT_ADDR
-				   NETSNMP_DS_LIB_TEMP_FILE_PATTERN
-				   NETSNMP_DS_LIB_AUTHMASTERKEY
-				   NETSNMP_DS_LIB_PRIVMASTERKEY
-				   NETSNMP_DS_LIB_AUTHLOCALIZEDKEY
-				   NETSNMP_DS_LIB_PRIVLOCALIZEDKEY
-				   NETSNMP_DS_LIB_APPTYPES
-				   NETSNMP_DS_LIB_KSM_KEYTAB
-				   NETSNMP_DS_LIB_KSM_SERVICE_NAME
-				   NETSNMP_DS_LIB_SBSM_LOCAL_PWD
 	netsnmp_ds_get_boolean
 	netsnmp_ds_get_int
 	netsnmp_ds_get_string
@@ -153,27 +124,11 @@ use vars qw(@ISA %EXPORT_TAGS @EXPORT_OK @EXPORT $VERSION $AUTOLOAD);
 				   NETSNMP_DS_LIB_HAVE_READ_PREMIB_CONFIG
 				   NETSNMP_DS_LIB_HAVE_READ_CONFIG
 				   NETSNMP_DS_LIB_QUICKE_PRINT
-				   NETSNMP_DS_LIB_DONT_PRINT_UNITS
-				   NETSNMP_DS_LIB_NO_DISPLAY_HINT
-				   NETSNMP_DS_LIB_16BIT_IDS
-				   NETSNMP_DS_LIB_DONT_PERSIST_STATE
-				   NETSNMP_DS_LIB_2DIGIT_HEX_OUTPUT
-				   NETSNMP_DS_LIB_IGNORE_NO_COMMUNITY
-				   NETSNMP_DS_LIB_DISABLE_PERSISTENT_LOAD
-				   NETSNMP_DS_LIB_DISABLE_PERSISTENT_SAVE
-				   NETSNMP_DS_LIB_APPEND_LOGFILES
-                                   NETSNMP_DS_LIB_CLIENT_ADDR_USES_PORT
 				   NETSNMP_DS_LIB_MIB_WARNINGS
 				   NETSNMP_DS_LIB_SECLEVEL
 				   NETSNMP_DS_LIB_SNMPVERSION
 				   NETSNMP_DS_LIB_DEFAULT_PORT
 				   NETSNMP_DS_LIB_OID_OUTPUT_FORMAT
-				   NETSNMP_DS_LIB_STRING_OUTPUT_FORMAT
-				   NETSNMP_DS_LIB_HEX_OUTPUT_LENGTH
-				   NETSNMP_DS_LIB_SERVERSENDBUF
-				   NETSNMP_DS_LIB_SERVERRECVBUF
-				   NETSNMP_DS_LIB_CLIENTSENDBUF
-				   NETSNMP_DS_LIB_CLIENTRECVBUF
 				   NETSNMP_DS_SNMP_VERSION_1
 				   NETSNMP_DS_SNMP_VERSION_2c
 				   NETSNMP_DS_SNMP_VERSION_3
@@ -189,40 +144,36 @@ use vars qw(@ISA %EXPORT_TAGS @EXPORT_OK @EXPORT $VERSION $AUTOLOAD);
 				   NETSNMP_DS_LIB_CONFIGURATION_DIR
 				   NETSNMP_DS_LIB_SECMODEL
 				   NETSNMP_DS_LIB_MIBDIRS
-				   NETSNMP_DS_LIB_OIDSUFFIX
-				   NETSNMP_DS_LIB_OIDPREFIX
-				   NETSNMP_DS_LIB_CLIENT_ADDR
-				   NETSNMP_DS_LIB_TEMP_FILE_PATTERN
-				   NETSNMP_DS_LIB_AUTHMASTERKEY
-				   NETSNMP_DS_LIB_PRIVMASTERKEY
-				   NETSNMP_DS_LIB_AUTHLOCALIZEDKEY
-				   NETSNMP_DS_LIB_PRIVLOCALIZEDKEY
-				   NETSNMP_DS_LIB_APPTYPES
-				   NETSNMP_DS_LIB_KSM_KEYTAB
-				   NETSNMP_DS_LIB_KSM_SERVICE_NAME
-				   NETSNMP_DS_LIB_SBSM_LOCAL_PWD
 );
-$VERSION = '5.0905';
+$VERSION = '5.1';
 
 sub AUTOLOAD {
     # This AUTOLOAD is used to 'autoload' constants from the constant()
-    # XS function.
+    # XS function.  If a constant is not found then control is passed
+    # to the AUTOLOAD in AutoLoader.
 
     my $constname;
     ($constname = $AUTOLOAD) =~ s/.*:://;
-    croak "&NetSNMP::default_store::constant not defined" if $constname eq 'cons
-tant';
-    my ($error, $val) = constant($constname);
-    if ($error) { croak $error; }
+    croak "& not defined" if $constname eq 'constant';
+    my $val = constant($constname, @_ ? $_[0] : 0);
+    if ($! != 0) {
+	if ($! =~ /Invalid/ || $!{EINVAL}) {
+	    $AutoLoader::AUTOLOAD = $AUTOLOAD;
+	    goto &AutoLoader::AUTOLOAD;
+	}
+	else {
+	    croak "Your vendor has not defined NetSNMP::default_store macro $constname";
+	}
+    }
     {
-        no strict 'refs';
-        # Fixed between 5.005_53 and 5.005_61
-#XXX    if ($] >= 5.00561) {
-#XXX        *$AUTOLOAD = sub () { $val };
-#XXX    }
-#XXX    else {
-            *$AUTOLOAD = sub { $val };
-#XXX    }
+	no strict 'refs';
+	# Fixed between 5.005_53 and 5.005_61
+#	if ($] >= 5.00561) {
+#	    *$AUTOLOAD = sub () { $val };
+#	}
+#	else {
+	    *$AUTOLOAD = sub { $val };
+#	}
     }
     goto &$AUTOLOAD;
 }
@@ -235,14 +186,15 @@ bootstrap NetSNMP::default_store $VERSION;
 
 1;
 __END__
+# Below is stub documentation for your module. You better edit it!
 
 =head1 NAME
 
-NetSNMP::default_store - Perl extension for Net-SNMP generic storage of global data
+NetSNMP::default_store - Perl extension for blah blah blah
 
 =head1 SYNOPSIS
 
-  use NetSNMP::default_store qw(:all);
+  use NetSNMP::default_store;
   $port = netsnmp_ds_get_int(NETSNMP_DS_LIBRARY_ID, DS_LIB_DEFAULT_PORT);
   netsnmp_ds_set_int(NETSNMP_DS_LIBRARY_ID, DS_LIB_DEFAULT_PORT, 161);
 
@@ -292,27 +244,11 @@ None by default.
 				   NETSNMP_DS_LIB_HAVE_READ_PREMIB_CONFIG
 				   NETSNMP_DS_LIB_HAVE_READ_CONFIG
 				   NETSNMP_DS_LIB_QUICKE_PRINT
-				   NETSNMP_DS_LIB_DONT_PRINT_UNITS
-				   NETSNMP_DS_LIB_NO_DISPLAY_HINT
-				   NETSNMP_DS_LIB_16BIT_IDS
-				   NETSNMP_DS_LIB_DONT_PERSIST_STATE
-				   NETSNMP_DS_LIB_2DIGIT_HEX_OUTPUT
-				   NETSNMP_DS_LIB_IGNORE_NO_COMMUNITY
-				   NETSNMP_DS_LIB_DISABLE_PERSISTENT_LOAD
-				   NETSNMP_DS_LIB_DISABLE_PERSISTENT_SAVE
-				   NETSNMP_DS_LIB_APPEND_LOGFILES
-                                   NETSNMP_DS_LIB_CLIENT_ADDR_USES_PORT
 				   NETSNMP_DS_LIB_MIB_WARNINGS
 				   NETSNMP_DS_LIB_SECLEVEL
 				   NETSNMP_DS_LIB_SNMPVERSION
 				   NETSNMP_DS_LIB_DEFAULT_PORT
 				   NETSNMP_DS_LIB_OID_OUTPUT_FORMAT
-				   NETSNMP_DS_LIB_STRING_OUTPUT_FORMAT
-				   NETSNMP_DS_LIB_HEX_OUTPUT_LENGTH
-				   NETSNMP_DS_LIB_SERVERSENDBUF
-				   NETSNMP_DS_LIB_SERVERRECVBUF
-				   NETSNMP_DS_LIB_CLIENTSENDBUF
-				   NETSNMP_DS_LIB_CLIENTRECVBUF
 				   NETSNMP_DS_SNMP_VERSION_1
 				   NETSNMP_DS_SNMP_VERSION_2c
 				   NETSNMP_DS_SNMP_VERSION_3
@@ -328,18 +264,6 @@ None by default.
 				   NETSNMP_DS_LIB_CONFIGURATION_DIR
 				   NETSNMP_DS_LIB_SECMODEL
 				   NETSNMP_DS_LIB_MIBDIRS
-				   NETSNMP_DS_LIB_OIDSUFFIX
-				   NETSNMP_DS_LIB_OIDPREFIX
-				   NETSNMP_DS_LIB_CLIENT_ADDR
-				   NETSNMP_DS_LIB_TEMP_FILE_PATTERN
-				   NETSNMP_DS_LIB_AUTHMASTERKEY
-				   NETSNMP_DS_LIB_PRIVMASTERKEY
-				   NETSNMP_DS_LIB_AUTHLOCALIZEDKEY
-				   NETSNMP_DS_LIB_PRIVLOCALIZEDKEY
-				   NETSNMP_DS_LIB_APPTYPES
-				   NETSNMP_DS_LIB_KSM_KEYTAB
-				   NETSNMP_DS_LIB_KSM_SERVICE_NAME
-				   NETSNMP_DS_LIB_SBSM_LOCAL_PWD
 
 =head2 Exportable functions
 
